<?php

if (!defined('_PS_VERSION_'))
{
    exit();
}
    
class KeyInvoice extends Module
{
    public function __construct()
    {
        $this->name = 'keyinvoice';
        $this->tab  = 'billing_invoicing';
        $this->version = '1.0.1';
        $this->author = 'T&T';
        $this->need_instance = 0;
        $this->ps_versions_compliance = array(
            'min' => '1.7', 
            'max' => _PS_VERSION_
        );
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('KeyInvoice');
        $this->description = $this->l('Sincronize a sua loja online com o software de faturação KeyInvoice.');
        $this->confirmUninstall = $this->l('Tem certeza que deseja desinstalar?');

        if (!Configuration::get('MYMODULE_NAME')) {     
		  $this->warning = $this->l('Nenhum nome foi fornecido');
	    }
    }

    public function install()
    {
        $this->setMenu('KeyInvoiceTab', $this->l('KeyInvoice'), (_PS_VERSION_ > 1.6) ? Tab::getIdFromClassName('SELL') : '0');
        //$this->setMenu('KeyInvoiceConfiguracao', $this->l('Configurações'), Tab::getIdFromClassName('KeyInvoiceTab'));
        $this->setMenu('KeyInvoiceProdutos', $this->l('Produtos'), Tab::getIdFromClassName('KeyInvoiceTab'));
        $this->setMenu('KeyInvoiceMovimentos', $this->l('Encomendas'), Tab::getIdFromClassName('KeyInvoiceTab'));
        $this->setMenu('KeyInvoiceAjuda', $this->l('Ajuda'), Tab::getIdFromClassName('KeyInvoiceTab'));

        if (!parent::install() ||
            !Configuration::updateValue('KEYINVOICE_API_KEY', '') ||
			!Configuration::updateValue('KEYINVOICE_DOCS', 0) ||
			!Configuration::updateValue('KEYINVOICE_TYPE', 0) ||
			!Configuration::updateValue('KEYINVOICE_API_URL', 'https://login.keyinvoice.com/API3_ws.php?wsdl') || 
			//!Configuration::updateValue('KEYINVOICE_LIMIT_CRON', 0) ||
            !Configuration::updateValue('KEYINVOICE_EMAIL', 0) ||
            !Configuration::updateValue('KEYINVOICE_ALLOW_FINAL_CONSUMER', 0) ||
            !Configuration::updateValue('KEYINVOICE_REF', '') ||
			!Configuration::updateValue('MYMODULE_NAME', 'keyinvoice')
		  )
        {
            return false;
        }
        
        Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."category` ADD  `keyinvoice_ref` VARCHAR(255) DEFAULT '' NOT NULL ");
		Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."manufacturer` ADD  `keyinvoice_ref` VARCHAR(255) DEFAULT '' NOT NULL ");
		Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."product` ADD  `keyinvoice_date_upd` DATETIME");
        Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."orders` ADD  `_keyinvoice_document_id` VARCHAR(255) DEFAULT '' ");

        $this->registerHook('actionAdminControllerSetMedia');
        $this->registerHook('displayAdminOrderSide');
        
        return true;
    }

    public function uninstall()
    {
        $this->delMenu('KeyInvoiceTab');
        $this->delMenu('KeyInvoiceProdutos');
        $this->delMenu('KeyInvoiceMovimentos');
        $this->delMenu('KeyInvoiceAjuda');
        
        Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."category` DROP COLUMN `keyinvoice_ref` ");
		Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."manufacturer` DROP COLUMN `keyinvoice_ref` ");
		Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."product` DROP COLUMN `keyinvoice_date_upd` ");
        Db::getInstance()->execute("ALTER TABLE `"._DB_PREFIX_."orders` DROP COLUMN  `_keyinvoice_document_id` ");

        $this->unregisterHook('actionAdminControllerSetMedia');
        $this->unregisterHook('displayAdminOrderSide');
        return parent::uninstall();
    }

    public function hookActionAdminControllerSetMedia()
    {
		$this->context->controller->addCSS($this->_path.'views/css/'.$this->name.'.css', 'all');
        $this->context->controller->addJS($this->_path.'views/js/'.$this->name.'.js');
    }

    public function hookDisplayAdminOrderSide($params)
    {
        $config = false;
        $api_key = Configuration::get('KEYINVOICE_API_KEY');
        $api_url = Configuration::get('KEYINVOICE_API_URL');

        if (($api_key != '') && ($api_url != ''))
        {
            $config = true;
        }
        
        $order = new Order($params['id_order']);

        $key = Db::getInstance()->getRow('SELECT `_keyinvoice_document_id`
            FROM `'._DB_PREFIX_.'orders`
            WHERE `id_order` = \''.pSQL($params['id_order']).'\'');

        return $this->get('twig')->render(
            '@Modules/keyinvoice/views/templates/admin/export.html.twig',
            [
                'img_path'=> _MODULE_DIR_.$this->name.'/logo2.png',
                'file_path'=> _MODULE_DIR_.$this->name.'/classes/Export.php',
                'order_id'=> $params['id_order'],
                'order_state'=> $order->getCurrentState(),
                'doc_key'=> $key['_keyinvoice_document_id'],
                'complete'=> $config
            ]
        );
    }

    public function setMenu($className, $text, $parent = '2')
    {
        $tab = new Tab();

        foreach (Language::getLanguages(false) as $lang)
        {
            $tab->name[(int)$lang['id_lang']] = $text;
        }

        $tab->class_name = $className;
        $tab->id_parent = $parent;
        $tab->module = $this->name;
        $tab->icon = 'receipt';
        $tab->add();

        unset($tab);
        return true;
    }

    public function delMenu($className)
    {
        $idTab = Tab::getIdFromClassName($className);

        if ((int)$idTab > 0) {
            $tab = new Tab($idTab);
            $tab->delete();
            return true;
        }
        return false;
    }

    public function getContent()
	{
		$output = null;

		if (Tools::isSubmit('submit'.$this->name))
		{
			$api_key = strval(Tools::getValue('KEYINVOICE_API_KEY'));
			$docs = intval(Tools::getValue('KEYINVOICE_DOCS'));
			$type = intval(Tools::getValue('KEYINVOICE_TYPE'));
			$api_url = strval(Tools::getValue('KEYINVOICE_API_URL'));
            $send_email = intval(Tools::getValue('KEYINVOICE_EMAIL'));
            $final_consumer = intval(Tools::getValue('KEYINVOICE_ALLOW_FINAL_CONSUMER'));
            $doc_ref = strval(Tools::getValue('KEYINVOICE_REF'));
			//$cron_lim = strval(Tools::getValue('KEYINVOICE_LIMIT_CRON'));
					
			/* POR FAZER */
			if (!$api_key
			  || empty($api_key)
			  || !Validate::isGenericName($api_key))
				$output .= $this->displayError($this->l('Valores de configuração inválidos.'));
			else
			{
				Configuration::updateValue('KEYINVOICE_API_KEY', $api_key);
				Configuration::updateValue('KEYINVOICE_DOCS', $docs);
				Configuration::updateValue('KEYINVOICE_TYPE', $type);
				Configuration::updateValue('KEYINVOICE_API_URL', $api_url);
                Configuration::updateValue('KEYINVOICE_EMAIL', $send_email);
                Configuration::updateValue('KEYINVOICE_ALLOW_FINAL_CONSUMER', $final_consumer);
                Configuration::updateValue('KEYINVOICE_REF', $doc_ref);
				//Configuration::updateValue('KEYINVOICE_LIMIT_CRON', $cron_lim);
				
				$output .= $this->displayConfirmation($this->l('Configurações Editadas'));
			}
		}
		return $output.$this->displayForm();
	}
	
	public function displayForm()
	{
    // Get default language
		$default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

		// Init Fields form array
		$fields_form[0]['form'] = array(
			'legend' => array(
				'title' => $this->l('Keyinvoice Configurações'),
			),
			'description' => $this->l('Configurações necessárias no php.ini: soap.wsdl_cache_enabled=1 e soap.wsdl_cache_ttl=(tempo em segundos que o ficheiro será guardado, no mínimo 5 horas)'),
			'input' => array(
				array(
					'type' => 'text',
					'label' => $this->l('URL API'),
					'name' => 'KEYINVOICE_API_URL',
					'size' => 50,
					'required' => true
				),
				array(
					'type' => 'text',
					'label' => $this->l('Chave API'),
                    'desc' => $this->l('Para obter estes dados entre na sua conta KEYINVOICE e clique em Configurações -» separador "API KeyInvoice"'),
					'name' => 'KEYINVOICE_API_KEY',
					'size' => 50,
					'required' => true
				),
				array(
				  'type' => 'select',                              // This is a <select> tag.
				  'label' => $this->l('Tipo de Documento'),         // The <label> for this <select> tag.
				  'desc' => $this->l('Exportação de Encomendas'),  // A help text, displayed right next to the <select> tag.
				  'name' => 'KEYINVOICE_DOCS',                     // The content of the 'id' attribute of the <select> tag.
				  'required' => true,                              // If set to true, this option must be set.
				  'options' => array(
				    'query' => array(
						  array(
							'id_option' => 4,                 // The value of the 'value' attribute of the <option> tag.
							'name' => $this->l('Faturas')              // The value of the text content of the  <option> tag.
						  ),
						  array(
							'id_option' => 32,
							'name' => $this->l('Faturas Simplificadas')
						  ),
                          array(
							'id_option' => 34,
							'name' => $this->l('Faturas Recibo')
						  ),
                          array(
							'id_option' => 13,
							'name' => $this->l('Encomenda')
						  ),
						),                           // $options contains the data itself.
					'id' => 'id_option',                           // The value of the 'id' key must be the same as the key for 'value' attribute of the <option> tag in each $options sub-array.
					'name' => 'name'                               // The value of the 'name' key must be the same as the key for the text content of the <option> tag in each $options sub-array.
				  )
				),
                array(
                    'type'=> 'switch',
                    'label'=> $this->l('Manter em Rascunho?'),
                    'desc' => $this->l('Indique se pretende que o documento seja exportado em modo rascunho ou que seja criado e finalizado automaticamente no seu programa de faturação'),
                    'name' => 'KEYINVOICE_TYPE',
                    'required'=> false,
                    'class' => 't',
                    'is_bool'=> false,
                    'values' => array(
                        array(
                            'id'=> 'indexation_on',
                            'value' => 1,
                            'label'=> $this->l('Sim')
                        ),
                        array(
                            'id'=> 'indexation_off',
                            'value' => 0,
                            'label'=> $this->l('Não')
                        ),
                    )
                ),
                array(
                    'type'=> 'switch',
                    'label'=> $this->l('Enviar documento por email?'),
                    'desc' => $this->l('Indique se pretende que o documento seja enviado por email para o cliente logo que seja exportado em modo finalizado. Esta opção não se aplica documentos exportados como rascunho nem a clientes que não indiquem o email no processo de encomenda. O envio é feito pelo programa de faturação pelo que terá de ter um email configurado (Configurações -» separador Servidor Email'),
                    'name' => 'KEYINVOICE_EMAIL',
                    'required'=> false,
                    'class' => 't',
                    'is_bool'=> false,
                    'values' => array(
                        array(
                            'id'=> 'email_on',
                            'value' => 1,
                            'label'=> $this->l('Sim')
                        ),
                        array(
                            'id'=> 'email_off',
                            'value' => 0,
                            'label'=> $this->l('Não')
                        ),
                    )
                ),
                array(
                    'type'=> 'switch',
                    'label'=> $this->l('Permitir facturas a consumidor final?'),
                    'desc' => $this->l('Indique se pretende que o documento seja criado associado ao Consumidor Final caso o NIF introduzido não seja válido (não aplicável a Encomendas)'),
                    'name' => 'KEYINVOICE_ALLOW_FINAL_CONSUMER',
                    'required'=> false,
                    'class' => 't',
                    'is_bool'=> false,
                    'values' => array(
                        array(
                            'id'=> 'final_on',
                            'value' => 1,
                            'label'=> $this->l('Sim')
                        ),
                        array(
                            'id'=> 'final_off',
                            'value' => 0,
                            'label'=> $this->l('Não')
                        ),
                    )
                ),
                array(
					'type' => 'text',
					'label' => $this->l('Referência de Documento'),
                    'desc' => $this->l('Se diferente de vazio, o campo DocRef no programa de facturação será preenchido com o prefixo que aqui preencher, seguido do número da encomenda exportada.'),
					'name' => 'KEYINVOICE_REF',
					'size' => 50,
					'required' => false
				),
				/*array(
					'type' => 'text',
					'label' => $this->l('Limite máx. Cron'),
					'name' => 'KEYINVOICE_LIMIT_CRON',
					'size' => 50,
					'required' => true
				)*/
			),
			'submit' => array(
				'title' => $this->l('Save'),
				'class' => 'button'
            )
		);
		 
		$helper = new HelperForm();
		 
		// Module, token and currentIndex
		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		 
		// Language
		$helper->default_form_language = $default_lang;
		$helper->allow_employee_form_lang = $default_lang;
		 
		// Title and toolbar
		$helper->title = $this->displayName;
		$helper->show_toolbar = true;        // false -> remove toolbar
		$helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper->submit_action = 'submit'.$this->name;
		$helper->toolbar_btn = array(
			'save' =>
			array(
				'desc' => $this->l('Save'),
				'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
				'&token='.Tools::getAdminTokenLite('AdminModules'),
			),
			'back' => array(
				'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
				'desc' => $this->l('Back to list')
			)
		);
		 
		// Load current value
		$helper->fields_value['KEYINVOICE_API_KEY'] = Configuration::get('KEYINVOICE_API_KEY');
		$helper->fields_value['KEYINVOICE_DOCS'] = Configuration::get('KEYINVOICE_DOCS');
		$helper->fields_value['KEYINVOICE_TYPE'] = Configuration::get('KEYINVOICE_TYPE');
		$helper->fields_value['KEYINVOICE_API_URL'] = Configuration::get('KEYINVOICE_API_URL');
        $helper->fields_value['KEYINVOICE_EMAIL'] = Configuration::get('KEYINVOICE_EMAIL');
        $helper->fields_value['KEYINVOICE_ALLOW_FINAL_CONSUMER'] = Configuration::get('KEYINVOICE_ALLOW_FINAL_CONSUMER');
        $helper->fields_value['KEYINVOICE_REF'] = Configuration::get('KEYINVOICE_REF');
		//$helper->fields_value['KEYINVOICE_LIMIT_CRON'] = Configuration::get('KEYINVOICE_LIMIT_CRON');

		return $helper->generateForm($fields_form);
	}
}